/**
 * ns_fregister.C: This file contains code for the NickServ FREGISTER function.
 *
 * Copyright © 2007 Pippijn van Steenhoven / The Ermyth Team
 * Rights to this code are as documented in COPYING.
 *
 *
 * Portions of this file were derived from sources bearing the following license:
 * Copyright © 2005-2007 William Pitcock, et al.
 * Rights to this code are as documented in doc/pod/license.pod.
 *
 * $Id: ns_fregister.C,v 1.9 2007/09/22 14:27:27 pippijn Exp $
 */

#include "atheme.h"
#include <libermyth.h>
#include <ermyth/module.h>
#include <account/myuser.h>
#include <account/mynick.h>

static char const rcsid[] = "$Id: ns_fregister.C,v 1.9 2007/09/22 14:27:27 pippijn Exp $";

REGISTER_MODULE ("nickserv/fregister", false, "The Ermyth Team <http://ermyth.xinutec.org>");

static void ns_cmd_fregister (sourceinfo_t *si, int parc, char *parv[]);

command_t const ns_fregister = { "FREGISTER", "Registers a nickname on behalf of another user.", PRIV_USER_FREGISTER, 20, ns_cmd_fregister };

E cmdvec ns_cmdtree;
E helpvec ns_helptree;

bool
_modinit (module *m)
{
  ns_cmdtree << ns_fregister;
  help_addentry (ns_helptree, "FREGISTER", "help/nickserv/fregister", NULL);

  return true;
}

void
_moddeinit ()
{
  ns_cmdtree >> ns_fregister;
  help_delentry (ns_helptree, "FREGISTER");
}

static void
ns_cmd_fregister (sourceinfo_t *si, int parc, char *parv[])
{
  myuser_t *mu;
  mynick_t *mn;
  char *account;
  char *pass;
  char *email;
  int i, uflags = 0;

  account = parv[0], pass = parv[1], email = parv[2];

  if (!account || !pass || !email)
    {
      command_fail (si, fault::needmoreparams, STR_INSUFFICIENT_PARAMS, "FREGISTER");
      command_fail (si, fault::needmoreparams, "Syntax: FREGISTER <account> <password> <email> [CRYPTPASS]");
      return;
    }

  for (i = 3; i < parc; i++)
    {
      if (!strcasecmp (parv[i], "CRYPTPASS"))
	uflags |= MU_CRYPTPASS;
      else if (!strcasecmp (parv[i], "HIDEMAIL"))
	uflags |= MU_HIDEMAIL;
      else if (!strcasecmp (parv[i], "NOOP"))
	uflags |= MU_NOOP;
      else if (!strcasecmp (parv[i], "NEVEROP"))
	uflags |= MU_NEVEROP;
    }

  if ((!(uflags & MU_CRYPTPASS) && strlen (pass) > 32) || strlen (email) >= EMAILLEN)
    {
      command_fail (si, fault::badparams, STR_INVALID_PARAMS, "FREGISTER");
      return;
    }

  if (!nicksvs.no_nick_ownership && IsDigit (*account))
    {
      command_fail (si, fault::badparams, "For security reasons, you can't register your UID.");
      return;
    }

  if (strchr (account, ' ') || strchr (account, '\n') || strchr (account, '\r') || account[0] == '=' || account[0] == '#' || account[0] == '@' || account[0] == '+' || account[0] == '%' || account[0] == '!' || strchr (account, ','))
    {
      command_fail (si, fault::badparams, "The account name \2%s\2 is invalid.", account);
      return;
    }

  if (!validemail (email))
    {
      command_fail (si, fault::badparams, "\2%s\2 is not a valid email address.", email);
      return;
    }

  /* make sure it isn't registered already */
  if (nicksvs.no_nick_ownership ? myuser_t::find (account) != NULL : mynick_t::find (account) != NULL)
    {
      command_fail (si, fault::alreadyexists, "\2%s\2 is already registered.", account);
      return;
    }

  mu = myuser_t::create (account, pass, email, uflags | config_options.defuflags | MU_NOBURSTLOGIN);
  mu->registered = NOW;
  mu->lastlogin = NOW;
  if (!nicksvs.no_nick_ownership)
    {
      mn = mynick_t::create (mu, mu->name);
      mn->registered = NOW;
      mn->lastseen = NOW;
    }

  snoop ("FREGISTER: \2%s\2 to \2%s\2 by \2%s\2", account, email, get_oper_name (si));
  logcommand (si, CMDLOG_REGISTER, "FREGISTER %s to %s", account, email);
  if (is_soper (mu))
    {
      wallops ("%s used FREGISTER on account \2%s\2 with services operator privileges.", get_oper_name (si), mu->name);
      snoop ("SOPER: \2%s\2", mu->name);
    }

  command_success_nodata (si, "\2%s\2 is now registered to \2%s\2.", mu->name, mu->email);
  mu->callback.registered (mu);
}
